#-----------------------------------------------------------------------------------------------------------------
# my.mer.sim function from Ben bolker to simulate data from a glmer
# (http://glmm.wdfiles.com/local--files/examples/glmmfuns.R)
#-----------------------------------------------------------------------------------------------------------------
my.mer.sim <- function (object, nsim = 1, seed = NULL, ...) 
{
    if (!is.null(seed)) 
        set.seed(seed)
    if (!exists(".Random.seed", envir = .GlobalEnv)) 
        runif(1)
    if (!is.null(object@call$offset))
      warning("offset ignored -- not yet implemented")
    RNGstate <- .Random.seed  
    dims <- object@dims
    sigma <- lme4:::sigma(object) 
    etasim.fix <- as.vector(object@X %*% fixef(object))  
    etasim.reff <- (as(t(object@A) %*% matrix(rnorm(nsim * dims["q"]), nc = nsim), 
                       "matrix"))  
    if (length(object@V) == 0 && length(object@muEta) == 0)  {
      etasim.resid <- matrix(rnorm(nsim * dims["n"]), nc = nsim) 
      etasim <- etasim.fix + sigma*(etasim.reff+etasim.resid)
      return(etasim)
    }
    if (length(object@muEta)>0) {
      etasim <- etasim.fix+etasim.reff 
      family <- object@call$family
      if(is.symbol(family)) family <- as.character(family)
      if(is.character(family))
        family <- get(family, mode = "function", envir = parent.frame(2))
      if(is.function(family)) family <- family()
      if(is.null(family$family)) stop("'family' not recognized")
      musim <- family$linkinv(etasim)
      n <- length(musim) 
      vsim <- switch(family$family,
                     poisson=rpois(n,lambda=musim),
                     quasipoisson=rqpois(n,musim,sigma), 
                     binomial={
                       resp <- model.response(object@frame);
                       if (!is.matrix(resp)) {
                         binom1 <- TRUE
                         rbinom(n,prob=musim,size=1)
                       } else {
                         binom1 <- FALSE
                         sizes <- rowSums(resp)
                         Y <- rbinom(n, size = sizes, prob = musim)
                         YY <- cbind(Y, sizes - Y)
                         colnames(YY) <- colnames(resp)
                         yy <- split(as.data.frame(YY),
                                     rep(1:nsim,each=length(sizes)))
                         yy <- as.data.frame(yy)
                       }
                     },
                     stop("simulation not implemented for family",
                          family$family))
      if (!(family$family=="binomial" && !binom1)) {
        vsim <- matrix(vsim,nc=nsim)
      }
      return(drop(vsim)) 
    }
    stop("simulate method for NLMMs not yet implemented")
  }
  
#------------------------------------------------------------------------------------------------------------------
# MCMC Bootstrapping
#------------------------------------------------------------------------------------------------------------------
glm.fitted.model=XX # where XX is name of the model fitted
glm.dataset=YY # where YY is the name of the original dataset
new.model=x2~ZZ #where ZZ is the model formula, i.e. the fixed and the random effects 

n=10000
ndigits=4
coefs = summary(glm.fitted.model)@coefs
ncoef = length(coefs[, 1])
vals=data.frame(array(NA,dim=c(n,ncoef)))
for (i in 1:n) {
	print(i)
	print(date())
	# simulate 0-1
    x2 = my.mer.sim(glm.fitted.model)
    # need to bond the new variable "x2" with the original dataset
    data.sim=cbind(glm.dataset, x2)
    # refit estimates
    glm.sim=glmer(new.model, data=data.sim, family="binomial")
    # save estimates
    vals[i,] = as.vector(glm.sim@fixef)
  }

#------------------------------------------------------------------------------------------------------------------
# Calculate 95% highest posterior density confidence interval  
#------------------------------------------------------------------------------------------------------------------
nr = nrow(vals)
prop = colSums(vals > 0)/nr
ans = 2 * pmax(0.5/nr, pmin(prop, 1 - prop))
HPDlower=as.vector(array(NA,dim=ncoef))
HPDupper=as.vector(array(NA,dim=ncoef))
for (j in 1:ncoef){
	HPDlower[j]=quantile(vals[,j],0.025)
	HPDupper[j]=quantile(vals[,j],0.975)
	}
fixed = data.frame(Estimate = round(as.numeric(coefs[,1]), ndigits), MCMCmean = round(apply(vals, 2, mean), ndigits), HPD95lower = round(HPDlower, ndigits), HPD95upper = round(HPDupper, ndigits), pMCMC = round(ans, ndigits), pT = round(as.numeric(coefs[,4]), ndigits), row.names = names(coefs[,1]))
colnames(fixed)[ncol(fixed)] = "Pr(>|t|)"
print(fixed)

